(function() {
  var RouterFactory;

  RouterFactory = (function() {
    var PRESET_ACTIONS, express, path, snakeCase;

    path = require('path');

    express = require('express');

    snakeCase = require('snake-case');

    PRESET_ACTIONS = {
      list: {
        method: 'GET'
      },
      create: {
        method: 'POST'
      },
      show: {
        method: 'GET',
        id: true
      },
      update: {
        method: 'PUT',
        id: true
      },
      remove: {
        method: 'DELETE',
        id: true
      }
    };

    function RouterFactory(config) {
      this._router = express.Router();
      this._controller = config.controller;
      this._resource = config.resource;
      this._enable(config.only || Object.keys(PRESET_ACTIONS));
      this._custom(config.custom || []);
    }

    RouterFactory.prototype._enable = function(actions) {
      var action, config, id, _i, _len, _results;
      this._presetRoutes = [];
      _results = [];
      for (_i = 0, _len = actions.length; _i < _len; _i++) {
        action = actions[_i];
        config = PRESET_ACTIONS[action];
        config.action = action;
        if (config.id) {
          id = ':id';
        } else {
          id = '';
        }
        config.route = path.resolve('/', this._resource, id);
        _results.push(this._presetRoutes.push(config));
      }
      return _results;
    };

    RouterFactory.prototype._custom = function(custom) {
      var config, routePath, _i, _len, _results;
      this._customRoutes = [];
      _results = [];
      for (_i = 0, _len = custom.length; _i < _len; _i++) {
        config = custom[_i];
        if (typeof config === 'string') {
          config = {
            action: config
          };
        }
        if (!config.method) {
          config.method = 'GET';
        }
        routePath = config.path || snakeCase(config.action).replace(/_/g, '-');
        config.route = path.resolve('/', this._resource, routePath);
        _results.push(this._customRoutes.push(config));
      }
      return _results;
    };

    RouterFactory.prototype.getRouter = function() {
      var config, method, routes, _i, _len;
      routes = [].concat(this._customRoutes).concat(this._presetRoutes);
      for (_i = 0, _len = routes.length; _i < _len; _i++) {
        config = routes[_i];
        method = config.method.toLowerCase();
        if (typeof this._controller[config.action] !== 'function') {
          throw new Error("Controller has no method '" + config.action + "'");
        }
        this._router[method](config.route, this._controller[config.action]);
      }
      return this._router;
    };

    RouterFactory.create = function(config) {
      require('./validator').validate(config);
      return (new RouterFactory(config)).getRouter();
    };

    return RouterFactory;

  })();

  module.exports = RouterFactory;

}).call(this);
